# coding: utf-8
# Copyright (c) 2016, 2025, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20240531


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class HttpEndpointAuthConfig(object):
    """
    Auth related information to be used when invoking external endpoint
    """

    #: A constant which can be used with the http_endpoint_auth_config_type property of a HttpEndpointAuthConfig.
    #: This constant has a value of "HTTP_ENDPOINT_NO_AUTH_CONFIG"
    HTTP_ENDPOINT_AUTH_CONFIG_TYPE_HTTP_ENDPOINT_NO_AUTH_CONFIG = "HTTP_ENDPOINT_NO_AUTH_CONFIG"

    #: A constant which can be used with the http_endpoint_auth_config_type property of a HttpEndpointAuthConfig.
    #: This constant has a value of "HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG"
    HTTP_ENDPOINT_AUTH_CONFIG_TYPE_HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG = "HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG"

    #: A constant which can be used with the http_endpoint_auth_config_type property of a HttpEndpointAuthConfig.
    #: This constant has a value of "HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG"
    HTTP_ENDPOINT_AUTH_CONFIG_TYPE_HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG = "HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG"

    #: A constant which can be used with the http_endpoint_auth_config_type property of a HttpEndpointAuthConfig.
    #: This constant has a value of "HTTP_ENDPOINT_IDCS_AUTH_CONFIG"
    HTTP_ENDPOINT_AUTH_CONFIG_TYPE_HTTP_ENDPOINT_IDCS_AUTH_CONFIG = "HTTP_ENDPOINT_IDCS_AUTH_CONFIG"

    def __init__(self, **kwargs):
        """
        Initializes a new HttpEndpointAuthConfig object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.generative_ai_agent.models.HttpEndpointIdcsAuthConfig`
        * :class:`~oci.generative_ai_agent.models.HttpEndpointDelegatedBearerAuthConfig`
        * :class:`~oci.generative_ai_agent.models.HttpEndpointNoAuthConfig`
        * :class:`~oci.generative_ai_agent.models.HttpEndpointOciResourcePrincipalAuthConfig`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param http_endpoint_auth_config_type:
            The value to assign to the http_endpoint_auth_config_type property of this HttpEndpointAuthConfig.
            Allowed values for this property are: "HTTP_ENDPOINT_NO_AUTH_CONFIG", "HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG", "HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG", "HTTP_ENDPOINT_IDCS_AUTH_CONFIG", 'UNKNOWN_ENUM_VALUE'.
            Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.
        :type http_endpoint_auth_config_type: str

        """
        self.swagger_types = {
            'http_endpoint_auth_config_type': 'str'
        }
        self.attribute_map = {
            'http_endpoint_auth_config_type': 'httpEndpointAuthConfigType'
        }
        self._http_endpoint_auth_config_type = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['httpEndpointAuthConfigType']

        if type == 'HTTP_ENDPOINT_IDCS_AUTH_CONFIG':
            return 'HttpEndpointIdcsAuthConfig'

        if type == 'HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG':
            return 'HttpEndpointDelegatedBearerAuthConfig'

        if type == 'HTTP_ENDPOINT_NO_AUTH_CONFIG':
            return 'HttpEndpointNoAuthConfig'

        if type == 'HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG':
            return 'HttpEndpointOciResourcePrincipalAuthConfig'
        else:
            return 'HttpEndpointAuthConfig'

    @property
    def http_endpoint_auth_config_type(self):
        """
        **[Required]** Gets the http_endpoint_auth_config_type of this HttpEndpointAuthConfig.
        The type of Auth config to be used when invoking an external endpoint.
        The allowed values are:
        - `HTTP_ENDPOINT_NO_AUTH_CONFIG`: Indicates that no authentication is required for invoking the endpoint.
        - `HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG`: Specifies Bearer Token Authentication, where the same Bearer token received as part of the Agent Chat
        API request is used to invoke the external endpoint.
        - `HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG`: Specifies authentication using Oracle Cloud Infrastructure (OCI) Resource Principal, leveraging
        OCI's identity-based authentication mechanism.
        - `HTTP_ENDPOINT_IDCS_AUTH_CONFIG`: Specifies authentication using Oracle Identity Cloud Service (IDCS), leveraging OAuth 2.0 for token-based authentication.

        Allowed values for this property are: "HTTP_ENDPOINT_NO_AUTH_CONFIG", "HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG", "HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG", "HTTP_ENDPOINT_IDCS_AUTH_CONFIG", 'UNKNOWN_ENUM_VALUE'.
        Any unrecognized values returned by a service will be mapped to 'UNKNOWN_ENUM_VALUE'.


        :return: The http_endpoint_auth_config_type of this HttpEndpointAuthConfig.
        :rtype: str
        """
        return self._http_endpoint_auth_config_type

    @http_endpoint_auth_config_type.setter
    def http_endpoint_auth_config_type(self, http_endpoint_auth_config_type):
        """
        Sets the http_endpoint_auth_config_type of this HttpEndpointAuthConfig.
        The type of Auth config to be used when invoking an external endpoint.
        The allowed values are:
        - `HTTP_ENDPOINT_NO_AUTH_CONFIG`: Indicates that no authentication is required for invoking the endpoint.
        - `HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG`: Specifies Bearer Token Authentication, where the same Bearer token received as part of the Agent Chat
        API request is used to invoke the external endpoint.
        - `HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG`: Specifies authentication using Oracle Cloud Infrastructure (OCI) Resource Principal, leveraging
        OCI's identity-based authentication mechanism.
        - `HTTP_ENDPOINT_IDCS_AUTH_CONFIG`: Specifies authentication using Oracle Identity Cloud Service (IDCS), leveraging OAuth 2.0 for token-based authentication.


        :param http_endpoint_auth_config_type: The http_endpoint_auth_config_type of this HttpEndpointAuthConfig.
        :type: str
        """
        allowed_values = ["HTTP_ENDPOINT_NO_AUTH_CONFIG", "HTTP_ENDPOINT_DELEGATED_BEARER_AUTH_CONFIG", "HTTP_ENDPOINT_OCI_RESOURCE_PRINCIPAL_AUTH_CONFIG", "HTTP_ENDPOINT_IDCS_AUTH_CONFIG"]
        if not value_allowed_none_or_none_sentinel(http_endpoint_auth_config_type, allowed_values):
            http_endpoint_auth_config_type = 'UNKNOWN_ENUM_VALUE'
        self._http_endpoint_auth_config_type = http_endpoint_auth_config_type

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
